From c96c1759e9e8fa908cf02c6f9efed341c4f10a93 Mon Sep 17 00:00:00 2001
From: Matt Langlois <fletchto99@gmail.com>
Date: Fri, 12 Jun 2020 12:57:34 -0600
Subject: [PATCH] When parsing cookies, only decode the values

Patch utils to fix cookie parsing

[CVE-2020-8184]
---
 lib/rack/utils.rb  | 8 ++++++--
 test/spec_utils.rb | 4 ++++
 2 files changed, 10 insertions(+), 2 deletions(-)

diff --git a/lib/rack/utils.rb b/lib/rack/utils.rb
index f033c90..d3b3b1d 100644
--- a/lib/rack/utils.rb
+++ b/lib/rack/utils.rb
@@ -212,8 +212,12 @@ module Rack
       # The syntax for cookie headers only supports semicolons
       # User Agent -> Server ==
       # Cookie: SID=31d4d96e407aad42; lang=en-US
-      cookies = parse_query(header, ';') { |s| unescape(s) rescue s }
-      cookies.each_with_object({}) { |(k, v), hash| hash[k] = Array === v ? v.first : v }
+      return {} unless header
+      header.split(/[;] */n).each_with_object({}) do |cookie, cookies|
+        next if cookie.empty?
+        key, value = cookie.split('=', 2)
+        cookies[key] = (unescape(value) rescue value) unless cookies.key?(key)
+      end
     end
 
     def add_cookie_to_header(header, key, value)
diff --git a/test/spec_utils.rb b/test/spec_utils.rb
index b39f4a0..273dc6c 100644
--- a/test/spec_utils.rb
+++ b/test/spec_utils.rb
@@ -524,6 +524,10 @@ describe Rack::Utils, "cookies" do
 
     env = Rack::MockRequest.env_for("", "HTTP_COOKIE" => "foo=bar").freeze
     Rack::Utils.parse_cookies(env).must_equal({ "foo" => "bar" })
+
+    env = Rack::MockRequest.env_for("", "HTTP_COOKIE" => "%66oo=baz;foo=bar")
+    cookies = Rack::Utils.parse_cookies(env)
+    cookies.must_equal({ "%66oo" => "baz", "foo" => "bar" })
   end
 
   it "adds new cookies to nil header" do
-- 
2.26.2

